/*  parmcheck:  a program to check the parameters stored in stune for */
/*              consistency and adherance to the rules and rules of   */
/*              thumb found in various performance books, and compare */
/*              stune and mtune rules. 11/27/94 DW                    */

/*  This is not meant to be incredibly elegant or efficient code,     */
/*  but it does the job and it's free.  The reason mtune and stune    */
/*  are searched is that someone may want to determine if their new   */
/*  parameters make sense before they rebuild their kernel.  The      */
/*  files are repeatedly searched because it was really easy to code  */
/*  that way.  Since this is unlikely to be run repeatedly and the    */
/*  software is free, it'll do for now (though it's likely to be the  */
/*  ugliest code on the skunkware CD :-).                             */


/*  (c) Copyright 1994-1995 by Donald R. Winterhalter                 */
/*  (c) Copyright 1995-1996 by Aurora Software, Plaistow, NH 03865    */
/*      all rights reserved.                                          */

#include <sys/types.h>
#include <sys/fcntl.h>
#include <stddef.h>
#include <stdio.h>
#include <stdlib.h>

long getparm();
long getbuf();
int prime();
int power2();
void suffix();
int rangecheck();
int cpus();
long getmem();
long getinod();
long getproc();

int main()
{
FILE *pptr;

int warn = 0;
int numcpus = 0;
long nbuf = 0;
long nhbuf = 0;
long nmpbuf = 0;
long ctbuf = 0;
long ninode = 0;
long nhinode = 0;
long bdflushr = 0;
long nautoup = 0;
long gpgslo = 0;
long gpgshi = 0;
long htcache = 0;
long hthash = 0;
long dtcache = 0;
long dthash = 0;
long nproc = 0;
long maxup = 0;
long memsize = 0;
long maxdisk = 0;
long maxproc = 0;
long maxinode = 0;
long maxfile = 0;
long maxregion = 0;
long maxflckrec = 0;

float bufratio = 0.0;
float inoderatio = 0.0;
float htratio = 0.0;
float dtratio = 0.0;
float procratio = 0.0;
long comment = 0;


char aa[1] = {' '};  /* nhbuf flag      */
char ab[1] = {' '};  /* ctbuf flag      */
char ac[1] = {' '};  /* maxdisk flag    */
char ad[1] = {' '};  /* nhinode flag    */
char ae[1] = {' '};  /* maxinode flag   */
char af[1] = {' '};  /* htcache flag    */
char ag[1] = {' '};  /* hthash flag     */
char ah[1] = {' '};  /* maxproc flag    */
char ai[1] = {' '};  /* maxfile flag    */
char aj[1] = {' '};  /* maxregion flag  */
char ak[1] = {' '};  /* nmpbuf flag     */
char al[1] = {' '};  /* maxflckrec flag */
char am[1] = {' '};  /* nbuf flag       */
char an[1] = {' '};  /* maxup flag      */
char ap[1] = {' '};  /* bdflushr flag   */
char aq[1] = {' '};  /* nautoup flag    */
char ar[1] = {' '};  /* gpgslo flag     */
char as[1] = {' '};  /* gpgshi flag     */
char at[1] = {' '};  /* dtcache flag    */
char au[1] = {' '};  /* dthash flag     */


/*  header information */
printf("Parmcheck v1.51 - A utility to check various tunable parameters\n\n");
/*  number of CPUs */
printf("System Environment\n");
printf("------------------\n");
numcpus = cpus(pptr);
printf("           # OF CPUS = %d\n",numcpus);
memsize = getmem(pptr);
if (memsize > 0.0)
	printf("           Memory size = %ld megabytes.\n",memsize);
else
	printf("           Memory size could not be determined\n");
printf("\n");	/* end of systems environment section */
/*  disks and buffers section */
printf("Buffers management\n");
printf("------------------\n");
nbuf = getbuf(pptr,am);			/* Get it from stune */
nhbuf = getparm(pptr,"NHBUF",aa);
nmpbuf = getparm(pptr,"NMPBUF",ak);
nhinode = getparm(pptr,"NHINODE",ad);
bdflushr = getparm(pptr,"BDFLUSHR",ap);
nautoup = getparm(pptr,"NAUTOUP",aq);
ctbuf = getparm(pptr,"CTBUFSIZE",ab);
ninode=getinod(pptr);
printf("           NBUF = %ld",nbuf);
suffix(am);
printf("           NHBUF = %ld",nhbuf);
suffix(aa);
printf("           NMPBUF = %ld",nmpbuf);
suffix(ak);
printf("           NHINODE = %ld",nhinode);
suffix(ad);
printf("           Grown size of inode table = %ld.\n",ninode);
printf("           BDFLUSHR = %ld",bdflushr);
suffix(ap);
printf("           NAUTOUP = %ld",nautoup);
suffix(aq);
printf("           CTBUFSIZE = %ld",ctbuf);
suffix(ab);
if (nbuf > 0)
	{
	printf("(Comment): NBUF size usually OK if sar -b %%rcache > ");
	printf("90%% and %%wcache > 65%%.\n");
	}
else
	printf("(Comment): NBUF size could not be determined.\n");
if (nhbuf > 0)
	bufratio = (float)nbuf / (float)nhbuf;
else
	bufratio = 0.00;
if (numcpus < 2)
	{
	if (bufratio >= 2.00)
		{
		printf("(Warning): NBUF divided by NHBUF should be less ");
		printf("than 2.\n");
		++warn;
		}
	else if (bufratio > 1.33)
		printf("(Comment): NBUF/NHBUF ratio of %.2f is OK.\n",bufratio);
	else if (bufratio > 0.00)
		{
		printf("(Comment): NBUF/NHBUF ratio of %.2f is OK,",bufratio);
		printf(" but the advantages of a large\n");
		printf("           NHBUF value may not be worth the memory ");
		printf("if your system is\n");
		printf("           memory-poor.\n");
		}
	else if (bufratio == 0.00)
		{
		printf("(Comment): NHBUF has been set to zero, and is ");
		printf("determined automatically\n");
		printf("           at startup.\n");
		}
	}

else if (numcpus > 1)
	{
	if (bufratio > 0.50)
		{
		printf("(Warning): With MPX, NHBUF should be a power of 2, ");
		printf("and at least twice the\n");
		printf("           value of NBUF.\n");
		++warn;
		}
	else if (bufratio > 0.25)
		printf("(Comment): NBUF/NHBUF ratio of %.2f is OK.\n",bufratio);
	else if (bufratio > 0.00)
		{
		printf("(Comment): NBUF/NHBUF ratio of %.2f is OK,",bufratio);
		printf(" but the advantages of a large\n");
		printf("           NHBUF value may not be worth the memory ");
		printf("if your system is\n");
		printf("           memory-poor.\n");
		}
	else if (bufratio == 0.00)
		{
		printf("(Comment): NHBUF has been set to zero, and is ");
		printf("determined automatically\n");
		printf("           at startup.\n");
		}
	}
if (power2(nhbuf) == 1)
	printf("(Comment): NHBUF correctly defined an a power of 2.\n");
else if (nhbuf > 0)
	{
	printf("(Warning): NHBUF should be zero or a power of 2.\n");
	++warn;
	}
warn += rangecheck("NHBUF",aa);
if (nmpbuf == 0)
	{
	printf("(Comment): NMPBUF size determined at startup.  See ");
	printf("page 86 of the\n");
	printf("           OpenServer Performance Guide for tuning ");
	printf("information.\n");
	}
else
	{
	printf("(Comment): For more information on tuning the NMPBUF ");
	printf("parameter, see page 86\n");
	printf("           of the OpenServer Performance Guide.\n");
	}
warn += rangecheck("NMPBUF",ak);
if (nhinode > 0)
	inoderatio = (float)ninode / (float)nhinode;
else
	inoderatio = 0.00;
if (power2(nhinode) == 1)
	{
	printf("(Comment): NHINODE correctly defined as a power ");
	printf("of 2.\n");
	}
else
	{
	printf("(Warning): NHINODE should be a power of 2.\n");
	++warn;
	}
if (numcpus < 2)
	{
	if (inoderatio > 2)
		{
		printf("(Warning): NHINODE should be a power of 2, ");
		printf("not be less than half the\n");
		printf("           grown size of the inode table.\n");
		++warn;
		}
	else
		{
		printf("(Comment): The grown size of the inode table ");
		printf("divided by NHINODE is\n");
		printf("           an acceptable %.2f.\n",inoderatio);
		}
	}
else if (numcpus > 1)
	{
	if (inoderatio > .5)
		{
		printf("(Warning): With SMP, NHINODE should be a power of ");
		printf("2, and greater than\n");
		printf("           twice the grown size of the inode ");
		printf("table.\n");
		++warn;
		}
	else
		{
		printf("(Comment): The grown size of the inode table ");
		printf("divided by NHINODE is\n");
		printf("           an acceptable %.2f.\n",inoderatio);
		}
	}
warn += rangecheck("NHINODE",ad);
if (bdflushr > 60 &&
    nautoup < 15)
	{
	printf("(Warning): BDFLUSHR has been set considerably higher ");
	printf("than NAUTOUP.\n");
	printf("           This results in buffers being quickly ");
	printf("marked as dirty,\n");
	printf("           but not written to disk in a reasonable ");
	printf("length of time.\n");
	++warn;
	comment = 1;
	}
if (bdflushr < 10)
	{
	printf("(Comment): BDFLUSHR has been set to a low value.  ");
	printf("This will increase\n");
	printf("           disk I/O, and decrease data lost in a "); 
	printf("system crash.\n");
	comment = 1;
	}
if (nautoup > 15)
	{
	printf("(Comment): NAUTOUP has been set to a high value.  ");
	printf("This will decrease\n");
	printf("           disk I/O, and increase data lost in a "); 
	printf("system crash.\n");
	comment = 1;
	}
if (nautoup < 10)
	{
	printf("(Comment): NAUTOUP has been set to a low value.  ");
	printf("This will increase\n");
	printf("           disk I/O, and decrease data lost in a "); 
	printf("system crash.\n");
	comment = 1;
	}
if (nautoup > 10 &&
    bdflushr > 30) 
	{
	printf("(Warning): Since both BDFLUSHR and NAUTOUP have ");
	printf("been set to values\n");
	printf("           higher than their defaults, significant "); 
	printf("amounts of data \n");
	printf("           may be lost in a system crash and a UPS ");
	printf("is recommended.\n");
	++warn;
	comment = 1;
	}
if (comment == 0)
	{
	printf("(Comment): BDFLUSHR and NAUTOUP values have not ");
	printf("triggered any special\n");
	printf("           comments or warnings.\n");
	}
warn += rangecheck("BDFLUSHR",ap);
warn += rangecheck("NAUTOUP",aq);
comment = 0;
warn += rangecheck("CTBUFSIZE",ab);
if (ctbuf > 128 &&
    ctbuf <= 256)
	{
	printf("(Comment): CTBUFSIZE OK if system has enough ");
	printf("memory.\n");
	}
else if (ctbuf == 128)
	{
	printf("(Comment): CTBUFSIZE OK, but a larger value might improve ");
	printf("non-SCSI tape\n");
	printf("           device performance if the system is not ");
	printf("memory-poor.\n");
	}
else if (ctbuf < 128)
	{
	printf("(Comment): CTBUFSIZE OK if system is memory-poor, ");
	printf("tape device performance\n");
	printf("           not critical, or tape device is SCSI.\n");
	}
printf("\n");	/* end of disks & buffers section */

/*  Processes and paging section */
printf("Processes and paging\n");
printf("--------------------\n");
gpgslo = getparm(pptr,"GPGSLO",ar);
gpgshi = getparm(pptr,"GPGSHI",as);
printf("           GPGSLO = %ld",gpgslo);
suffix(ar);
printf("           GPGSHI = %ld",gpgshi);
suffix(as);
if (gpgslo > memsize * 32 &&   
    gpgshi > memsize * 51)    
	{
	printf("(Comment): GPGSLO and GPGSHI have been set well ");
	printf("above normal recommendations.\n");
	printf("           If the system is paging, these values ");
	printf("may be too high.\n");
	comment = 1;
	}
if ((float)gpgshi > (float)(gpgslo * 2) &&
    (gpgshi - gpgslo) > 50)
	{
	printf("(Comment): The difference between GPGSLO and GPGSHI ");
	printf("is large.  This may cause\n");
	printf("           occasional I/O bottlenecks while dirty ");
	printf("pages are written to disk.\n");
	comment = 1;
	}
if ((float)gpgshi < (float)(gpgslo * 1.2) &&
    (gpgshi - gpgslo) < 50)
	{
	printf("(Comment): The difference between GPGSLO and GPGSHI ");
	printf("is small.  This may cause\n");
	printf("           the vhand daemon to be constantly active, ");
	printf("creating a CPU bottleneck.\n");
	comment = 1;
	}
if (comment == 0)
	{
	printf("(Comment): GPGSLO and GPGSHI values have not ");
	printf("triggered any special comments\n");
	printf("           or warnings.\n");
	}
comment = 0;
warn += rangecheck("GPGSLO",ar);
warn += rangecheck("GPGSHI",as);
printf("\n");	/* end of processes and paging section */

/*  Name cache section */
printf("Name cache\n");
printf("----------\n");
htcache = getparm(pptr,"HTCACHEENTS",af);
hthash = getparm(pptr,"HTHASHQS",ag);
dtcache = getparm(pptr,"DTCACHEENTS",at);
dthash = getparm(pptr,"DTHASHQS",au);
printf("           HTCACHEENTS = %ld",htcache);
suffix(af);
printf("           HTHASHQS = %ld",hthash);
suffix(ag);
printf("           DTCACHEENTS = %ld",dtcache);
suffix(at);
printf("           DTHASHQS = %ld",dthash);
suffix(au);
printf("(Comment): HTCACHENTS size OK if sar -n %%Hhit ratio > 90 ");
printf("percent.\n");
if (prime(hthash) == 1)
	{
	printf("(Comment): HTHASHQS correctly defined as a prime ");
	printf("number.\n");
	}
else
	{
	printf("(Warning): HTHASHQS should be a prime number.\n");
	++warn;
	}
if (hthash > 0)
	htratio = (float)htcache / (float)hthash;
else
	htratio = 0.00;
if (dthash > 0)
	dtratio = (float)dtcache / (float)dthash;
else
	dtratio = 0.00;
if (numcpus < 2)
	{
	if (htratio >= 2.33)
		{
		printf("(Warning): HTCACHEENTS should be about twice the ");
		printf("value of HTHASHQS.\n");
		++warn;
		}
	else if (htratio >= 0.50)
		{
		printf("(Comment): HTCACHENTS/HTHASHQS ratio of ");
		printf("%.2f is OK.\n",htratio);
		}
	else if (htratio > 0.00)
		/* HTHASHQS are 8 bytes each, so this message */
		/* isn't triggered unless they're way overused */
		{
		printf("(Comment): HTCACHEENTS/HTHASHQS ratio of ");
		printf("%.2f is OK, but the advantages of a \n",htratio);
		printf("           large HTHASHQS value may not be worth it ");
		printf("if your system is\n");
		printf("           memory-poor.\n");
		}
	}
if (numcpus > 1)
	{
	if (htratio > 1.0)
		{
		printf("(Warning): With SMP, HTHASHQS should be a prime ");
		printf("number slightly greater than\n");
		printf("           the value of HTCACHEENTS.\n");
		++warn;
		}
	else if (htratio > 0.33)
		{
		printf("(Comment): HTCACHENTS/HTHASHQS ratio of ");
		printf("%.2f is OK.\n",htratio);
		}
	else if (htratio > 0.00)
		/* HTHASHQS are 8 bytes each, so this message */
		/* isn't triggered unless they're way overused */
		{
		printf("(Comment): HTCACHEENTS/HTHASHQS ratio of ");
		printf("%.2f is OK, but the advantages of a \n",htratio);
		printf("           large HTHASHQS value may not be worth it ");
		printf("if your system is\n");
		printf("           memory-poor.\n");
		}
	}
warn += rangecheck("HTCACHEENTS",af);
warn += rangecheck("HTHASHQS",ag);
printf("(Comment): DTCACHENTS size OK if sar -n %%Dhit ratio > 90 ");
printf("percent.\n");
if (prime(dthash) == 1)
	{
	printf("(Comment): DTHASHQS correctly defined as a prime ");
	printf("number.\n");
	}
else
	{
	printf("(Warning): DTHASHQS should be a prime number.\n");
	++warn;
	}
if (numcpus < 2)
	{
	if (dtratio >= 2.33)
		{
		printf("(Warning): DTCACHEENTS should be about twice the ");
		printf("value of DTHASHQS.\n");
		++warn;
		}
	else if (dtratio >= 0.50)
		{
		printf("(Comment): DTCACHENTS/DTHASHQS ratio of ");
		printf("%.2f is OK.\n",dtratio);
		}
	else if (dtratio > 0.00)
		/* DTHASHQS are 8 bytes each, so this message */
		/* isn't triggered unless they're way overused */
		{
		printf("(Comment): DTCACHEENTS/DTHASHQS ratio of ");
		printf("%.2f is OK, but the advantages of a \n",dtratio);
		printf("           large DTHASHQS value may not be worth it ");
		printf("if your system is\n");
		printf("           memory-poor.\n");
		}
	}
if (numcpus > 1)
	{
	if (dtratio > 1.0)
		{
		printf("(Warning): With SMP, DTHASHQS should be a prime ");
		printf("number slightly greater than\n");
		printf("           the value of DTCACHEENTS.\n");
		++warn;
		}
	else if (dtratio > 0.33)
		{
		printf("(Comment): DTCACHENTS/DTHASHQS ratio of ");
		printf("%.2f is OK.\n",dtratio);
		}
	else if (dtratio > 0.00)
		/* DTHASHQS are 8 bytes each, so this message */
		/* isn't triggered unless they're way overused */
		{
		printf("(Comment): DTCACHEENTS/DTHASHQS ratio of ");
		printf("%.2f is OK, but the advantages of a \n",dtratio);
		printf("           large DTHASHQS value may not be worth it ");
		printf("if your system is\n");
		printf("           memory-poor.\n");
		}
	}
warn += rangecheck("DTCACHEENTS",at);
warn += rangecheck("DTHASHQS",au);
printf("\n");	/* end of name cache */

/*  User and group configuration section */
printf("User and group configuration\n");
printf("----------------------------\n");
maxup = getparm(pptr,"MAXUP",an);
nproc=getproc(pptr);
printf("           MAXUP = %ld",maxup);
suffix(an);
printf("           Grown size of process table = %ld.\n",nproc);
if (maxup > 0)
	procratio = (float)nproc / (float)maxup;
else
	procratio = 0.00;
if (maxup > 1000)
	{
	printf("(Comment): MAXUP has been set to an unusually ");
	printf("large value.\n");
	}
if (procratio > 1.10)
	{
	printf("(Comment): MAXUP is at least 10 percent smaller than ");
	printf("the grown size of the \n");
	printf("           process table, as recommended in ");
	printf("the OpenServer Performance Guide.\n");
	}
else
	{
	if (maxup < 250)
		{
		printf("(Comment): As a rule, MAXUP should be at least ");
		printf("10 percent smaller than the\n");
		printf("           grown size of the process table.  This ");
		printf("is not likely to matter\n");
		printf("           on a system where the value of MAXUP is ");
		printf("relatively small.\n");
		}
	else 
		{
		printf("(Comment): MAXUP should be at least 10 percent ");
		printf("smaller than the grown size\n");
		printf("           of the process table.  Please verify ");
		printf("that MAXUP needs to be set\n");
		printf("           this high.\n");
		}
	}
warn += rangecheck("MAXUP",an);
printf("\n");	/* end of user and group configuration section */

/*  Table limits section */
printf("Table limits\n");
printf("------------\n");

maxdisk = getparm(pptr,"MAX_DISK",ac);
maxinode = getparm(pptr,"MAX_INODE",ae);
maxproc = getparm(pptr,"MAX_PROC",ah);
maxfile = getparm(pptr,"MAX_FILE",ai);
maxregion = getparm(pptr,"MAX_REGION",aj);
maxflckrec = getparm(pptr,"MAX_FLCKREC",al);
printf("           MAX_DISK = %ld",maxdisk);
suffix(ac);
printf("           MAX_INODE = %ld",maxinode);
suffix(ae);
printf("           MAX_PROC = %ld",maxproc);
suffix(ah);
printf("           MAX_FILE = %ld",maxfile);
suffix(ai);
printf("           MAX_REGION = %ld",maxregion);
suffix(aj);
printf("           MAX_FLCKREC = %ld",maxflckrec);
suffix(al);

if (maxdisk == 0)
	{
	printf("(Comment): MAX_DISK has been set to zero, allowing ");
	printf("the number of disk drives\n");
	printf("           to be determined dynamically.\n");
	}
else
	{
	printf("(Comment): MAX_DISK has been set to a non-zero value.");
	printf("  If diskinfo table\n");
	printf("           overflows are seen, set this parameter ");
	printf("to zero.\n");
	}
warn += rangecheck("MAX_DISK",ac);
if (maxinode == 0)
	{
	printf("(Comment): MAX_INODE has been set to zero, allowing ");
	printf("the size of the inode\n");
	printf("           table to grow dynamically.\n");
	}
else
	{
	printf("(Comment): MAX_INODE should be at least 10-25%% larger ");
	printf("than the largest\n");
	printf("           inod-sz value seen in sar -v during peak loads.\n");
	}
warn += rangecheck("MAX_INODE",ae);
if (maxproc == 0)
	{
	printf("(Comment): MAX_PROC has been set to zero, allowing ");
	printf("the size of the process\n");
	printf("           table to grow dynamically.\n");
	}
else
	{
	printf("(Comment): MAX_PROC should be at least 10-25%% larger ");
	printf("than the largest\n");
	printf("           proc-sz value seen in sar -v during peak loads.\n");
	}
warn += rangecheck("MAX_PROC",ah);
if (maxfile == 0)
	{
	printf("(Comment): MAX_FILE has been set to zero, allowing ");
	printf("the size of the open\n");
	printf("           file table to grow dynamically.\n");
	}
else
	{
	printf("(Comment): MAX_FILE should be at least 10-25%% larger than ");
	printf("the largest file-sz\n");
	printf("           value seen in sar -v during peak loads.\n");
	}
warn += rangecheck("MAX_FILE",ai);
if (maxregion == 0)
	{
	printf("(Comment): MAX_REGION has been set to zero, allowing ");
	printf("the size of the\n");
	printf("           region table to grow dynamically.\n");
	}
if (maxregion > 0 &&
    maxproc > 0)
	{
	if (maxregion < (3 * maxproc) + 20)
		{
		printf("(Warning): MAX_REGION should be at least 3 times ");
		printf("the value of MAX_PROC, plus 20.\n");
		++warn;
		}
	else
		{
		printf("(Comment): MAX_REGION is sufficiently larger ");
		printf("than MAX_PROC.\n");
		}
	}
warn += rangecheck("MAX_REGION",aj);
if (maxflckrec == 0)
	{
	printf("(Comment): MAX_FLCKREC has been set to zero, allowing ");
	printf("the size of the lock\n");
	printf("           table to grow dynamically.\n");
	}
else
	{
	printf("(Comment): MAX_FLCKREC should be at least 10-25%% larger ");
	printf("than the largest\n");
	printf("           lock-sz value seen in sar -v during peak loads.\n");
	}
warn += rangecheck("MAX_FLCKREC",al);
printf("\nThere are %d warning(s) in the messages above.\n",warn);
/* footer sales pitch */
printf("\n");
printf("Copyright 1995-1996 Aurora Software Inc., All rights reserved.\n\n");
printf("This software is provided without warranty of any kind, either ");
printf("expressed or\n");
printf("implied.  This software is free and may be distributed freely ");
printf("throughout the\n");
printf("SCO UNIX community, provided that it is reproduced in its entirety, ");
printf("including\n");
printf("this notice.  Parmcheck version 1.5 is designed to work with SCO ");
printf("OpenServer\n");
printf("Release 5 only.  For support or information about this, or any of ");
printf("our other\n");
printf("inexpensive UNIX performance tools, call us at +1-603-382-4200 ");
printf("or use our\n");
printf("CompuServe address: 74013,1625 (full address: ");
printf("74013.1625@compuserve.com).\n");
return 0;
}

int power2(long a)
{
double d;
if (a > 0)
	for (d=a; d>=1; d/=2)
		if (d == 1)
			{
			return 1;
			}
return 0;
}
int prime(long a)
{
long k;
long prime = 0;
if (a < 4)
	return 1;
while(prime == 0)
	for (k=2; (k*k)<=a; ++k)
		{
		prime = a % k;
		if (prime == 0)
			return 0;
		}
return 1;
}

long getparm(FILE *fp, char c[], char l[1])
/* this function gets the value of a parameter from stune if   */
/* it's there, and from mtune if it's not.  It also returns    */
/* the value of the parameter, and sets the following flag in  */
/* the char l:                                                 */
/*	m - found in mtune                                     */
/*	s - found in stune, within allowed mtune values        */
/*	a - found in stune, above allowed mtune values         */
/*	b - found in stune, below allowed mtune values         */
{
char bufs[80];
char bufm[80];
char gr[80];
long val = 0;
long min = 0;
long max = 0;
l[0] = ' ';
bufs[0] = bufm[0] = '\0';
sprintf(gr, "cat /etc/conf/cf.d/stune | grep '%s'",c);
fp = popen(gr,"r");
fgets(bufs, 80, fp);
sprintf(gr, "cat /etc/conf/cf.d/mtune | grep '^%s'",c);
fp = popen(gr,"r");
fgets(bufm, 80, fp);
if (bufs[0] != '\0')
	{
	sscanf(bufs,"%*s%ld",&val);
	sscanf(bufm,"%*s%*d%ld%ld",&min,&max);
	if (val < min)
		l[0] = 'b';
	if (val > max)
		l[0] = 'a';
	if (l[0] == ' ')
		l[0] = 's';
	}
else
	{
	sscanf(bufm,"%*s%ld%ld%ld",&val,&min,&max);
	l[0] = 'm';
	}
return val;
}
long getbuf(FILE *fp, char l[1])
/*	o - found in messages                                  */
/*	s - found in stune, within allowed mtune values        */
/*	x - could not be determined                            */
{
char buf[80];
char gr[80];
long val = 0;
buf[0] = '\0';
sprintf(gr, "cat /etc/conf/cf.d/stune | grep 'NBUF'");
fp = popen(gr,"r");
fgets(buf, 80, fp);
if (buf[0] != '\0')
	{
	sscanf(buf,"%*s%ld",&val);
	if (val == 0)
		buf[0] = '\0';
	else
		l[0] = 's';
	}
if (buf[0] == '\0')
	{
	l[0] = 'm';	
	sprintf(gr,"tail -30 /usr/adm/messages|grep 'kernel:'");
	fp = popen(gr,"r");
	fgets(buf, 80, fp);
	sscanf(buf,"%*s%*s%*s%*s%ldk",&val);
	if (val == 0)
		{
		sprintf(gr,"tail -60 /usr/adm/messages|grep 'kernel:'");
		fp = popen(gr,"r");
		fgets(buf, 80, fp);
		sscanf(buf,"%*s%*s%*s%*s%ldk",&val);
		}
	}
if (val == 0)
	l[0] = 'x';
return val;
}
void suffix(char l[1])
{
if (l[0] == 'm')
	printf(" (default parameter).\n");
if (l[0] == 's' ||
    l[0] == 'a' || 
    l[0] == 'b') 
	printf(" (explicitly defined).\n");
if (l[0] == 'x')
	printf(" (cannot be determined).\n");
if (l[0] == 'o')
	printf(" (set by the operating system).\n");
}
int rangecheck(char c[], char aa[1])
{
if (aa[0] == 'a')
	{
	printf("(Warning): %s value exceeds the recommended ",c);
	printf("maximum in mtune.\n");
	return 1;
	}
if (aa[0] == 'b')
	{
	printf("(Warning): %s value is below the recommended ",c);
	printf("minimum in mtune.\n");
	return 1;
	}
return 0;
}
int cpus(FILE *fp)
{
/*    This function finds the number of CPUs present.     */
/*    If there's a problem, a value of 0 is returned.     */

int n = 0;
char buf[80];
	if ((fp = popen("uname -X | grep \"NumCPU\"","r")) != NULL)
		while (fgets(buf, 80, fp) != NULL)
			sscanf(buf,"NumCPU = %d",&n);
return n;
}
long getmem(FILE *fp)
{
char buf[200];
char gr[80];
char fld[25][6] = {'\0','\0','\0','\0','\0',
		   '\0','\0','\0','\0','\0',
		   '\0','\0','\0','\0','\0',
		   '\0','\0','\0','\0','\0',
		   '\0','\0','\0','\0','\0'};
long val = 0;
short n;
buf[0] = '\0';
sprintf(gr,"tail -30 /usr/adm/messages|grep 'mem:'");
fp = popen(gr,"r");
fgets(buf, 200, fp);
sscanf(buf,"%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s",
	   fld[0], fld[1], fld[2], fld[3], fld[4],
	   fld[5], fld[6], fld[7], fld[8], fld[9],
	   fld[10], fld[11], fld[12], fld[13], fld[14],
	   fld[15], fld[16], fld[17], fld[18], fld[19],
	   fld[20], fld[21], fld[22], fld[23], fld[24]);
n = 0;
if (strncmp(fld[0],"mem:",4) == 0)
	{
	sscanf(fld[3],"%ldk,",&val);
	}
else while (strncmp(fld[n++],"mem:",4) && n < 22)
	sscanf(fld[n+3],"%ldk",&val);
if (val == 0)
	{
	sprintf(gr,"tail -60 /usr/adm/messages|grep 'mem:'");
	fp = popen(gr,"r");
	fgets(buf, 200, fp);
	sscanf(buf,"%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s",
		   fld[0], fld[1], fld[2], fld[3], fld[4],
		   fld[5], fld[6], fld[7], fld[8], fld[9],
		   fld[10], fld[11], fld[12], fld[13], fld[14],
		   fld[15], fld[16], fld[17], fld[18], fld[19],
	  	   fld[20], fld[21], fld[22], fld[23], fld[24]);
	n = 0;
	if (strncmp(fld[0],"mem:",4) == 0)
		{
		sscanf(fld[3],"%ldk,",&val);
		}
	else while (strncmp(fld[n++],"mem:",4) && n < 22)
		sscanf(fld[n+3],"%ldk",&val);
	}
if (val == 0)
	{
	sprintf(gr,"tail -120 /usr/adm/messages|grep 'mem:'");
	fp = popen(gr,"r");
	fgets(buf, 200, fp);
	sscanf(buf,"%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s",
		   fld[0], fld[1], fld[2], fld[3], fld[4],
		   fld[5], fld[6], fld[7], fld[8], fld[9],
		   fld[10], fld[11], fld[12], fld[13], fld[14],
		   fld[15], fld[16], fld[17], fld[18], fld[19],
	  	   fld[20], fld[21], fld[22], fld[23], fld[24]);
	n = 0;
	if (strncmp(fld[0],"mem:",4) == 0)
		{
		sscanf(fld[3],"%ldk,",&val);
		}
	while (strncmp(fld[n++],"mem:",4) && n < 22)
		sscanf(fld[n+3],"%ldk",&val);
	}
if (val > 0)
	val = (float)((float)val+512.0)/1024;
return val;
}
long getinod(FILE *fp)
{
char bufs[80];
char bufm[80];
char gr[80];
long val = 0;
bufs[0] = bufm[0] = '\0';
sprintf(gr, "getconf KERNEL_INODE");
fp = popen(gr,"r");
fgets(bufs, 80, fp);
sscanf(bufs,"%ld",&val);
return val;
}
long getproc(FILE *fp)
{
char bufs[80];
char bufm[80];
char gr[80];
long val = 0;
bufs[0] = bufm[0] = '\0';
sprintf(gr, "getconf KERNEL_PROC");
fp = popen(gr,"r");
fgets(bufs, 80, fp);
sscanf(bufs,"%ld",&val);
return val;
}
